import "core-js/modules/es6.string.link";
import "core-js/modules/es6.regexp.to-string";
import _objectSpread from "@babel/runtime/helpers/objectSpread";
import "core-js/modules/es6.regexp.search";
import "core-js/modules/es6.string.ends-with";
import "core-js/modules/es6.object.assign";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import "regenerator-runtime/runtime";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _defineProperty from "@babel/runtime/helpers/defineProperty";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import React from 'react';
import { SheetsRegistry } from 'react-jss/lib/jss';
import { flushChunkNames } from 'react-universal-component/server';
import PWA from './PWA';
import createMemoryHistory from 'history/createMemoryHistory';
import { Helmet } from "react-helmet";
import { renderHtml, renderInitialStateScript, renderScript, renderStyle } from './renderers';
import getStats from 'react-storefront-stats';
import { renderAmpAnalyticsTags } from './Track';

var Server =
/*#__PURE__*/
function () {
  /**
   * @param {Object} config
   * @param {Object} config.theme A material-UI theme
   * @param {Class} config.model The model class for the root of the state tree
   * @param {React.Component} config.App The root app component
   * @param {Router} config.router An instance of moov_router's Router class
   * @param {Boolean} [config.deferScripts=true] Adds the defer attribute to all script tags to speed up initial page render. Defaults to true.
   * @param {Function} transform A function to transform the rendered HTML before it is sent to the browser
   */
  function Server(_ref) {
    var _this = this;

    var theme = _ref.theme,
        model = _ref.model,
        App = _ref.App,
        router = _ref.router,
        _ref$deferScripts = _ref.deferScripts,
        deferScripts = _ref$deferScripts === void 0 ? true : _ref$deferScripts,
        transform = _ref.transform;

    _classCallCheck(this, Server);

    _defineProperty(this, "serve",
    /*#__PURE__*/
    function () {
      var _ref2 = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee(request, response) {
        var state;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                console.error = console.error || console.log;
                console.warn = console.warn || console.log;

                if (!request.headers['x-rsf-routes']) {
                  _context.next = 4;
                  break;
                }

                return _context.abrupt("return", response.json(_this.router.routes.map(function (route) {
                  return route.path.spec;
                })));

              case 4:
                _context.prev = 4;
                // indicate to the XDN that we want to to add set the x-moov-cache-hit cookie so we can differentiate
                // cache hits and misses when tracking performance
                response.set('x-rsf-track-cache-hit', 'true');
                _context.next = 8;
                return _this.router.runAll(request, response);

              case 8:
                state = _context.sent;

                if (!(!state.proxyUpstream && !response.headersSent)) {
                  _context.next = 12;
                  break;
                }

                _context.next = 12;
                return _this.renderPWA({
                  request: request,
                  response: response,
                  state: state
                });

              case 12:
                _context.next = 18;
                break;

              case 14:
                _context.prev = 14;
                _context.t0 = _context["catch"](4);
                _context.next = 18;
                return _this.renderError(_context.t0, request, response);

              case 18:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[4, 14]]);
      }));

      return function (_x, _x2) {
        return _ref2.apply(this, arguments);
      };
    }());

    console.error = console.warn = console.log;
    Object.assign(this, {
      theme: theme,
      model: model,
      App: App,
      router: router,
      deferScripts: deferScripts,
      transform: transform
    });
  }
  /**
   * Handles an isomorphic request by serving json, html, or amp content based on the URL.
   */


  _createClass(Server, [{
    key: "setContentType",

    /**
     * Sets the content type to application/json for json URLs, text/html for all others
     * @param {Object} request 
     * @param {Response} response 
     */
    value: function setContentType(request, response) {
      if (response.get('content-type') == null) {
        if (request.path.endsWith('.json')) {
          response.set('content-type', 'application/json');
        } else {
          response.set('content-type', 'text/html');
        }
      }
    }
    /**
     * Renders either a JSON or HTML response for the given state based on the path suffix.
     * @param {Object} options
     * @param {Object} options.request The current request object
     * @param {Response} options.response The current response object
     * @param {Object} options.state The root react element
     * @return The html for app
     */

  }, {
    key: "renderPWA",
    value: function () {
      var _renderPWA = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee2(_ref3) {
        var request, response, state, protocol, hostname, port, path, search, amp, App, theme, sheetsRegistry, history, model, stats, html, helmet;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                request = _ref3.request, response = _ref3.response, state = _ref3.state;
                console.error = console.error || console.log;
                console.warn = console.warn || console.log;
                protocol = request.protocol, hostname = request.hostname, port = request.port, path = request.path, search = request.search;
                this.setContentType(request, response);

                if (!path.endsWith('.json')) {
                  _context2.next = 7;
                  break;
                }

                return _context2.abrupt("return", response.send(JSON.stringify(state)));

              case 7:
                amp = path.endsWith('.amp');
                App = this.App, theme = this.theme;
                sheetsRegistry = new SheetsRegistry();
                history = createMemoryHistory({
                  initialEntries: [path + search]
                });
                model = this.model.create(_objectSpread({}, state, {
                  amp: amp,
                  initialWidth: amp ? 'xs' : state.initialWidth,
                  location: _objectSpread({}, history.location, {
                    port: port,
                    protocol: protocol,
                    hostname: hostname
                  })
                }));
                this.chunkNames = [];
                _context2.prev = 13;
                _context2.next = 16;
                return getStats();

              case 16:
                stats = _context2.sent;
                html = renderHtml({
                  component: React.createElement(PWA, null, React.createElement(App, null)),
                  providers: {
                    app: model,
                    history: history,
                    analytics: {},
                    router: this.router
                  },
                  registry: sheetsRegistry,
                  theme: theme
                });
                helmet = Helmet.renderStatic();
                html = "\n        <!DOCTYPE html>\n        <html ".concat(helmet.htmlAttributes.toString(), ">\n          <head>\n            ").concat(helmet.title.toString(), "\n            <noscript id=\"jss-insertion-point\"></noscript>\n            ").concat(helmet.meta.toString(), "\n            ").concat(helmet.link.toString(), "\n            ").concat(helmet.script.toString(), "\n          </head>\n          <body ").concat(helmet.bodyAttributes.toString(), ">\n            ").concat(renderStyle({
                  registry: sheetsRegistry,
                  id: 'ssr-css'
                }), "\n            <noscript>\n              You need to enable JavaScript to run this app.\n            </noscript>\n            <div id=\"root\">").concat(html, "</div>\n            ").concat(amp ? '' : "\n              ".concat(renderInitialStateScript({
                  state: model,
                  defer: this.deferScripts
                }), "\n              ").concat(renderScript({
                  stats: stats,
                  chunk: 'bootstrap',
                  defer: this.deferScripts
                }), "\n              ").concat(this.getScripts(stats), "\n              ").concat(renderScript({
                  stats: stats,
                  chunk: 'main',
                  defer: this.deferScripts
                }), "\n            "), "\n          </body>\n        </html>\n      ");

                if (!(typeof this.transform === 'function')) {
                  _context2.next = 24;
                  break;
                }

                _context2.next = 23;
                return this.transform(html, {
                  model: model,
                  sheetsRegistry: sheetsRegistry,
                  helmet: helmet
                });

              case 23:
                html = _context2.sent;

              case 24:
                response.send(html);
                _context2.next = 32;
                break;

              case 27:
                _context2.prev = 27;
                _context2.t0 = _context2["catch"](13);
                // flush head content to prevent memory leak, see https://github.com/nfl/react-helmet#server-usage
                Helmet.renderStatic(); // flush amp analytics tags to prevent memory leak

                renderAmpAnalyticsTags();
                throw _context2.t0;

              case 32:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this, [[13, 27]]);
      }));

      function renderPWA(_x3) {
        return _renderPWA.apply(this, arguments);
      }

      return renderPWA;
    }()
    /**
     * Renders an error response, either as JSON or SSR HTML, depending on the suffix
     * on the request path.
     * @param {Error} e 
     * @param {Response} response 
     */

  }, {
    key: "renderError",
    value: function renderError(e, request, response) {
      response.status(500, 'error');
      return this.renderPWA({
        request: request,
        response: response,
        state: {
          page: 'Error',
          error: e.message,
          stack: process.env.MOOV_ENV === 'production' ? null : e.stack
        }
      });
    }
    /**
     * Gets the script tags that should added to the document based on the chunks used
     * to render the current request.
     * @param {Object} stats 
     * @return {String[]}
     */

  }, {
    key: "getScripts",
    value: function getScripts(stats) {
      var _this2 = this;

      return flushChunkNames(stats).map(function (chunk) {
        return renderScript({
          stats: stats,
          chunk: chunk,
          defer: _this2.deferScripts
        });
      }).filter(function (e) {
        return !!e;
      });
    }
  }]);

  return Server;
}();

export { Server as default };
//# sourceMappingURL=Server.js.map